from .tsinghua import TSingHuaDrive
from .tsinghua import download as download_tsinghua
from .tianchi import TianChiDrive
from .oss import OSSDrive
from .opendatalab import OpenDataLabDrive
from .lanzou import LanZouDrive, LanZouSnapshot
from .wenshushu import WSSDrive
from .alipan import AlipanDrive
from .os import OSDrive
